<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://speechkit.io
 * @since      1.0.0
 *
 * @package    Speechkit
 * @subpackage Speechkit/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Speechkit
 * @subpackage Speechkit/admin
 * @author     Kostas Vaggelakos <kostas@speechkit.io>
 */
class Speechkit_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;


	/**
	 * Status constants
	 */
	const STATUS_NEW        = 'New';
	const STATUS_UPDATED    = 'Updated';
	const STATUS_PROCESSED  = 'Processed';
	const STATUS_PROCESSING = 'Processing';
	const STATUS_ERROR      = 'Error';
	const STATUS_FAILED     = 'Failed';
	const STATUS_SCHEDULED     = 'Scheduled';

	/**
	 * Number of retries.
	 */
	const RETRIES = 15;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$options = get_option(
			'speechkit_settings',
			array(
				'speechkit_api_key' => '',
				'speechkit_id'      => '',
			)
		);

		$this->plugin_name = $plugin_name;
		// Using old version as a default value. All 2.5+ version will have correct value present in DB, so we need to show older version just for those old installs.
		$this->version = $version;
		$this->api     = new SpeechkitAPI(
			$this->version,
			$options['speechkit_api_key'],
			'', // sk_backend
			$options['speechkit_id']
		);
	}

	/**
	 * Get all post types
	 *
	 * @return array
	 **/
	public static function sk_available_get_post_types() {
		$default_ignore = array( 'attachment', 'revision', 'nav_menu_item', 'custom_css', 'customize_changeset', 'oembed_cache' );
		return array_diff( get_post_types(), $default_ignore );
	}

	/**
	 * Get allowed post types
	 *
	 * @return array
	 **/
	public static function sk_get_enabled_post_types() {
		$options = get_option(
			'speechkit_settings',
			array(
				'speechkit_select_post_types' => array( 'post' ),
			)
		);

		if ( ! isset( $options['speechkit_select_post_types'] ) || empty( $options['speechkit_select_post_types'] ) ) {
			return array();
		} elseif ( ! is_array( $options['speechkit_select_post_types'] ) ) {
			$options['speechkit_select_post_types'] = array( $options['speechkit_select_post_types'] );
		}

		return $options['speechkit_select_post_types'];
	}

	public function add_menu() {
		add_options_page( __( 'SpeechKit', 'speechkit' ), __( 'SpeechKit', 'speechkit' ), 'manage_options', 'speechkit', array( $this, 'create_admin_interface' ) );
	}

	public function add_meta_box() {
		$post_types = self::sk_get_enabled_post_types();
		if ( ! empty( $post_types ) ) {
			add_meta_box(
				'speechkit',
				__( 'SpeechKit', 'speechkit' ),
				array( $this, 'create_admin_meta_box' ),
				$post_types,
				'side',
				'default'
			);
		}
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @param string $hook Page hook
	 * @return void
	 * @since    1.0.0
	 */
	public function enqueue_styles( $hook ) {
		// Always add the metabox css, because we don't know where it's added
		wp_enqueue_style( $this->plugin_name . '-metabox', plugin_dir_url( __FILE__ ) . 'css/speechkit-admin-metabox.css', array(), $this->version, 'all' );
	}

	public function setup_cron_schedules( $schedules ) {
		if ( ! isset( $schedules['1min'] ) ) {
			$schedules['1min'] = array(
				'interval' => 1 * 60,
				'display'  => __( 'Once every minute', 'speechkit' ),
			);
		}
		return $schedules;
	}

    public function add_settings_link_to_plugin_page( $links ) {
        $links[] = '<a href="' .
            admin_url( 'options-general.php?page=speechkit' ) .
            '">' . __( 'Settings', 'speechkit' ) . '</a>';
        return $links;
    }

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @param string $hook Page hook
	 * @return void
	 * @since    1.0.0
	 */
	public function enqueue_scripts( $hook ) {
		wp_register_script( $this->plugin_name . '-metabox', plugin_dir_url( __FILE__ ) . 'js/speechkit-admin-metabox-2.5.0.js', array( 'jquery', 'underscore' ), $this->version, true );

		$selected_post_types   = self::sk_get_enabled_post_types();
		$selected_post_types[] = 'post-new';

		if ( in_array( str_replace( '.php', '', $hook ), apply_filters( 'spk_allowed_post_types', $selected_post_types ) ) ) {
			wp_enqueue_script( $this->plugin_name . '-metabox' );
			wp_localize_script(
				$this->plugin_name . '-metabox',
				'speechKitData',
				array(
					'error'       => __( "Something went wrong. This is probably SpeechKit's fault. Please try again later", 'speechkit' ),
					'enabled'     => __( 'Hide Audio Player', 'speechkit' ),
					'disabled'    => __( 'Show Audio Player', 'speechkit' ),
					'proccessing' => __( 'Proccesing Audio', 'speechkit' ),
					'tpl'         => array(
						'Error'      => $this->tpl_error(),
						'New'        => $this->tpl_new(),
						'Processed'  => $this->tpl_processed(),
						'Processing' => $this->tpl_processing(),
					),
					'gutenberg'   => ( function_exists( 'is_gutenberg_page' ) && is_gutenberg_page() ),
				)
			);
		}
	}

    public function init_speechkit_block() {
        wp_enqueue_script(
            'speechkit-block',
            plugin_dir_url(__FILE__) . 'js/speechkit-admin.js?v='.time(),
            array('wp-blocks','wp-editor'),
            true
        );
    }

	public function create_admin_interface() {
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/speechkit-admin-display.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/speechkit-admin-support.php';
	}

	public function create_admin_meta_box() {
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/speechkit-admin-metabox.php';
	}

	public function settings_init() {
		register_setting( 'speechkit', 'speechkit_settings' );

		add_settings_section(
			'speechkit_settings_section',
			__( 'Basic Settings', 'speechkit' ),
			array( $this, 'speechkit_settings_section_callback' ),
			'speechkit'
		);

		add_settings_section(
			'speechkit_advanced_settings_section',
			__( 'Advanced Settings', 'speechkit' ),
			array( $this, 'speechkit_advanced_settings_section_callback' ),
			'speechkit'
		);

		add_settings_field(
			'speechkit_enable',
			__( 'Enable SpeechKit', 'speechkit' ),
			array( $this, 'speechkit_enable_render' ),
			'speechkit',
			'speechkit_settings_section'
		);

        add_settings_field(
            'speechkit_disable_autopublish',
            __( 'Disable Auto-Publish', 'speechkit' ),
            array( $this, 'speechkit_disable_autopublish_render' ),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_enable_marfeel_comp',
            __( 'Enable Marfeel compatibility', 'speechkit' ),
            array( $this, 'speechkit_enable_marfeel_comp_render' ),
            'speechkit',
            'speechkit_settings_section'
        );

		add_settings_field(
			'speechkit_id',
			__( 'Project ID', 'speechkit' ),
			array( $this, 'speechkit_id_render' ),
			'speechkit',
			'speechkit_settings_section'
		);

		add_settings_field(
			'speechkit_api_key',
			__( 'API Key', 'speechkit' ),
			array( $this, 'speechkit_api_key_render' ),
			'speechkit',
			'speechkit_settings_section'
		);

		add_settings_field(
			'speechkit_merge_excerpt',
			__( 'Merge article excerpt with body', 'speechkit' ),
			array( $this, 'speechkit_merge_excerpt_render' ),
			'speechkit',
			'speechkit_advanced_settings_section'
		);

        add_settings_field(
            'speechkit_hcolumn',
            __( 'Hide "SpeechKit status" column', 'speechkit' ),
            array( $this, 'speechkit_hcolumn_render' ),
            'speechkit',
            'speechkit_advanced_settings_section'
        );

		add_settings_field(
			'speechkit_no_cron',
			__( 'Cron setting', 'speechkit' ),
			array( $this, 'speechkit_no_cron_render' ),
			'speechkit',
			'speechkit_advanced_settings_section'
		);

		add_settings_field(
			'speechkit_select_post_types',
			__( 'Post types', 'speechkit' ),
			array( $this, 'speechkit_select_post_types_render' ),
			'speechkit',
			'speechkit_advanced_settings_section'
		);
	}

	/**
	 * Basic Settings section callback
	 *
	 * @return void
	 * @since 2.5.0
	 **/
	public function speechkit_settings_section_callback() {
	}

	/**
	 * Advanced Settings section callback
	 *
	 * @return void
	 * @since 2.5.3
	 **/
	public function speechkit_advanced_settings_section_callback() {
	}

	/**
	 * Render Enable Speechkit setting
	 *
	 * @return void
	 * @since 2.5.0
	 **/
	public function speechkit_enable_render() {
		$options = get_option(
			'speechkit_settings',
			array(
				'speechkit_enable' => 1,
			)
		);
		if ( isset( $options['speechkit_enable'] ) ) {
			$option = $options['speechkit_enable'];
		} else {
			$option = 0;
		}
		?>
		<input type='checkbox' name='speechkit_settings[speechkit_enable]' <?php checked( $option, 1 ); ?> value='1'>
		<?php
	}

    /**
     * Render Hide column setting
     *
     * @return void
     * @since 2.6.7
     **/
    public function speechkit_hcolumn_render() {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_hcolumn' => 0,
            )
        );
        if ( isset( $options['speechkit_hcolumn'] ) ) {
            $option = $options['speechkit_hcolumn'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_hcolumn]' <?php checked( $option, 1 ); ?> value='1'>
        <?php
    }

    /**
     * Render Disable Autopublish Speechkit setting
     *
     * @return void
     * @since 2.6.2
     **/
    public function speechkit_disable_autopublish_render() {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_disable_autopublish' => 0,
            )
        );
        if ( isset( $options['speechkit_disable_autopublish'] ) ) {
            $option = $options['speechkit_disable_autopublish'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_disable_autopublish]' <?php checked( $option, 1 ); ?> value='1'>
        <?php _e( 'Disable auto-publish to generate audio for selected posts.' ); ?>
        <?php
    }

    /**
     * Render Enable Marfeel Compatibility setting
     *
     * @return void
     * @since 2.6.1
     **/
    public function speechkit_enable_marfeel_comp_render() {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_enable_marfeel_comp' => 1,
            )
        );
        if ( isset( $options['speechkit_enable_marfeel_comp'] ) ) {
            $option = $options['speechkit_enable_marfeel_comp'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_enable_marfeel_comp]' <?php checked( $option, 1 ); ?> value='1'>
        <?php
    }

	/**
	 * Render Project ID Speechkit setting
	 *
	 * @return void
	 * @since 2.5.0
	 **/
	public function speechkit_id_render() {
		$options = get_option( 'speechkit_settings' );
		if ( isset( $options['speechkit_id'] ) ) {
			$option = $options['speechkit_id'];
		} else {
			$option = '';
		}
		?>
		<input type='text' name='speechkit_settings[speechkit_id]' value='<?php echo $option; ?>'>
		<?php
	}

	/**
	 * Render API key setting
	 *
	 * @return void
	 * @since 2.5.0
	 **/
	public function speechkit_api_key_render() {
		$options = get_option( 'speechkit_settings' );
		if ( isset( $options['speechkit_api_key'] ) ) {
			$option = $options['speechkit_api_key'];
		} else {
			$option = '';
		}
		?>
		<input type='text' name='speechkit_settings[speechkit_api_key]' value='<?php echo $option; ?>'>
		<?php
	}

	/**
	 * Render Merge Excerpt setting
	 *
	 * @return void
	 * @since 2.5.0
	 **/
	public function speechkit_merge_excerpt_render() {
		$options = get_option( 'speechkit_settings' );
		if ( isset( $options['speechkit_merge_excerpt'] ) ) {
			$option = $options['speechkit_merge_excerpt'];
		} else {
			$option = 0;
		}
		?>
		<input type='checkbox' name='speechkit_settings[speechkit_merge_excerpt]' <?php checked( $option, 1 ); ?> value='1'>
		<?php
	}

	/**
	 * Render Use Cron checkbox
	 *
	 * @return void
	 * @since 2.5.3
	 **/
	public function speechkit_no_cron_render() {
		$options = get_option( 'speechkit_settings' );
		if ( isset( $options['speechkit_no_cron'] ) ) {
			$option = $options['speechkit_no_cron'];
		} else {
			$option = 0;
		}
		?>
		<input type='checkbox' name='speechkit_settings[speechkit_no_cron]' <?php checked( $option, 1 ); ?> value='1'>
		<?php _e( 'Disable Cron and send post text to API immediately on post publish.' ); ?>
		<p class="description">
			<?php _e( 'Normally posts are processed with a separate cron task. Check this if you have disabled Cron or there are isses with running Cron on your server.', 'speechkit' ); ?>
		</p>
		<?php
	}

	/**
	 * Render Select Post Types setting
	 *
	 * @return void
	 * @since 2.5.0
	 **/
	public function speechkit_select_post_types_render() {
		$options = get_option(
			'speechkit_settings',
			array(
				'post',
			)
		);
		if ( ! isset( $options['speechkit_select_post_types'] ) || empty( $options['speechkit_select_post_types'] ) ) {
			$options['speechkit_select_post_types'] = array();
		}
		?>
		<select name='speechkit_settings[speechkit_select_post_types][]' multiple="multiple">
			<?php
			foreach ( self::sk_available_get_post_types() as $key => $post_type ) {
				echo '<option value="' . $post_type . '" ' . selected( in_array( $post_type, $options['speechkit_select_post_types'] ) , true ) . '>' . $post_type . '</option>'; // phpcs:ignore
			}
			?>
		</select>
		<?php
	}

	/**
	 * Get theme and plugins info
	 *
	 * @return array
	 * @since 2.5.0
	 **/
	public static function get_environment() {
		$data = array(
			'wpversion'      => get_bloginfo( 'version' ),
			'options'        => wp_load_alloptions(),
			'active_plugins' => array(),
			'active_theme'   => array(),
			'posts'          => array(),
			'phpversion'     => phpversion(),
		);

		$active_plugins_array = get_option( 'active_plugins' );

		if ( ! empty( $active_plugins_array ) ) {
			foreach ( get_plugins() as $plugin_slug => $plugin ) {
				if ( in_array( $plugin_slug, $active_plugins_array ) ) {
					$data['active_plugins'][] = $plugin;
				}
			}
		}

		$data['active_theme'] = wp_get_theme();

        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_sp_posts' => 0,
            )
        );

        if( isset($options['speechkit_sp_posts']) ) {

            $args = array(
                'post_type' => self::sk_available_get_post_types(),
                'posts_per_page' => 3,
                'cache_results' => false,
                'update_post_term_cache' => false,
                'update_post_meta_cache' => false,
                'no_found_rows' => true,
                'fields' => 'ids',
                'meta_query' => array(
                    array(
                        'key' => 'speechkit_status',
                        'compare' => 'EXISTS',
                    ),
                ),
            );
            $posts = new WP_Query($args);
            if ($posts->have_posts()) {
                foreach ($posts->posts as $key => $post_id) {
                    $data['posts'][$key]['id'] = $post_id;
                    $data['posts'][$key]['custom'] = get_post_custom($post_id);
                }
            }
        }

		return apply_filters( 'spk_environment_data', $data );
	}

	/**
	 * Print Admin Notices
	 *
	 * @return void
	 * @since 2.5.0
	 */
	public function print_plugin_admin_notices() {
		if ( isset( $_REQUEST['sk_form_support'] ) ) { // phpcs:ignore
			if ( 'success' === $_REQUEST['sk_form_support'] ) { // phpcs:ignore
				echo '<div class="notice notice-success is-dismissible"><p>' . __( 'Your email was send. We will contact you shortly.', 'speechkit' ) . '</p></div>'; // phpcs:ignore
			} else {
				echo '<div class="notice notice-error is-dismissible"><p>' . __( 'There was an error while sending your email.', 'speechkit' ) . '</p></div>'; // phpcs:ignore
			}
		}
	}

	public static function ensure_endbled() {
		$options = get_option(
			'speechkit_settings',
			array(
				'speechkit_enable' => 1,
			)
		);

		return $options['speechkit_enable'] == 1;
	}

    public static function is_disabled_autopublish() {
        $options = get_option('speechkit_settings');
        return isset($options['speechkit_disable_autopublish']);
    }

	public static function ensure_settings() {
		$options = get_option(
			'speechkit_settings',
			array(
				'speechkit_api_key' => '',
				'speechkit_id'      => '',
			)
		);

		return ( $options['speechkit_api_key'] && $options['speechkit_id'] );
	}

    public function future_post_hook( $post_id, $post )
    {
		if ( in_array( $post->post_type, self::sk_get_enabled_post_types() ) ) {
			self::publish_post_hook( $post_id, $post, 'scheduled' );
		}
    }

	public function draft_post_hook( $post_id, $post )
	{
		if (in_array($this->get_status($post_id), array(self::STATUS_PROCESSED, self::STATUS_ERROR, self::STATUS_PROCESSING, self::STATUS_FAILED))) {
			update_post_meta($post_id, 'publish_post_to_speechkit', true);
		} elseif (!isset($_POST['publish_post_to_speechkit']) || $_POST['publish_post_to_speechkit'] != 1) {
			update_post_meta($post_id, 'publish_post_to_speechkit', -1);
			return;
		} else {
			update_post_meta($post_id, 'publish_post_to_speechkit', true);
		}
	}

	/**
	 * On private post gone visible update audio.
	 *
	 * @param int $post_id Post ID.
	 * @param object $post Post Object.
	 *
	 * @return void
	 * @since 2.5.8
	 **/
	public function publish_private_post_hook( $post_id, $post ) {
		if ( in_array( $post->post_type, self::sk_get_enabled_post_types() ) ) {
			self::publish_post_hook( $post_id, $post );
		}
	}

	/**
	 * On post update or creation mark it to be processed later, or process right away if cron is disabled.
	 *
	 * @param int $post_id Post ID.
	 * @param object $post Post Object.
	 *
	 * @return void
	 **/
	public function publish_post_hook( $post_id, $post = false, $state = false ) {
		$options = get_option( 'speechkit_settings' );
		if ( isset( $options['speechkit_no_cron'] ) ) {
			$cron_disabled = $options['speechkit_no_cron'];
		} else {
			$cron_disabled = 0;
		}

		//Check if has auto-publish setting
        $publish_post_to_speechkit = get_post_meta( $post_id, 'publish_post_to_speechkit', true );
        if( in_array($this->get_status( $post_id ), array(self::STATUS_PROCESSED, self::STATUS_ERROR, self::STATUS_SCHEDULED, self::STATUS_PROCESSING, self::STATUS_FAILED)) ||
            ( $publish_post_to_speechkit && $publish_post_to_speechkit == 1 ) ){
            update_post_meta($post_id, 'publish_post_to_speechkit', true);
        }elseif (!isset($_POST['publish_post_to_speechkit']) || $_POST['publish_post_to_speechkit'] != 1) {
            update_post_meta($post_id, 'publish_post_to_speechkit', -1);
            return;
        } else {
            update_post_meta($post_id, 'publish_post_to_speechkit', true);
        }

        if( !$post ){
			$post = get_post($post_id);
        }
		$hash = $this->get_post_hash( $post );

		switch ( $this->get_status( $post_id ) ) {
			case self::STATUS_PROCESSED:
				if ( get_post_meta( $post_id, '_speechkit_text', true ) !== $hash ) {
					update_post_meta( $post_id, 'speechkit_retries', 0 );
					update_post_meta( $post_id, '_speechkit_text', $hash );
					$this->set_status( $post_id, self::STATUS_UPDATED );
					if ( $cron_disabled ) {
						$this->process_post( $post_id, false, $state );
					} else {
						if ( ! wp_next_scheduled( 'check_speechkit_status_action' ) ) {
							$this->schedule_status_check();
						}
					}
				}
				break;
			case self::STATUS_SCHEDULED:
				$post_st = get_post_status( $post_id );
                update_post_meta( $post_id, 'speechkit_retries', 0 );
                update_post_meta( $post_id, '_speechkit_text', $hash );
                $this->set_status( $post_id, self::STATUS_UPDATED );
                if( $post_st == 'publish' ){
                    $state = 'unprocessed';
                }else{
					$state = 'scheduled';
                }
                $this->process_post( $post_id, true, $state );
                //if ( !$cron_disabled && $post_st == 'publish' ) {
                    if ( ! wp_next_scheduled( 'check_speechkit_status_action' ) ) {
                        $this->schedule_status_check();
                    }
                //}
				break;
			case self::STATUS_FAILED:
			case self::STATUS_ERROR:
				if ( get_post_meta( $post_id, '_speechkit_text', true ) !== $hash ) {
					update_post_meta( $post_id, 'speechkit_retries', 0 );
					update_post_meta( $post_id, '_speechkit_text', $hash );
					$this->set_status( $post_id, self::STATUS_PROCESSING );
					if ( $cron_disabled ) {
						$this->process_post( $post_id, false, $state );
					} else {
						if ( ! wp_next_scheduled( 'check_speechkit_status_action' ) ) {
							$this->schedule_status_check();
						}
					}
				}
				break;
			case self::STATUS_PROCESSING:
					if ( get_post_meta( $post_id, '_speechkit_text', true ) !== $hash ) {
						$this->process_post( $post_id, true, $state );
					}
				break;
			default:
				update_post_meta( $post_id, '_speechkit_text', $hash );
				if ( ! $this->get_status( $post_id ) ) {
					$this->set_status( $post_id, self::STATUS_NEW );
				}
				$this->process_post( $post_id, false, $state );
				if ( $cron_disabled ) {
					$this->process_post( $post_id, false, $state );
				} else {
					if ( ! wp_next_scheduled( 'check_speechkit_status_action' ) ) {
						$this->schedule_status_check();
					}
				}
				break;
		}
	}

	/**
	 * Get hash of post title, post excerpt and post content.
	 *
	 * @param object $post Post object
	 * @return string
	 * @since 2.5.3
	 **/
	public function get_post_hash( $post ) {
		$title   = apply_filters( 'the_title', $post->post_title );
		$excerpt = '';
		$options = get_option( 'speechkit_settings' );
		if ( isset( $options['speechkit_merge_excerpt'] ) && $options['speechkit_merge_excerpt'] == 1 ) {
			$excerpt = apply_filters( 'the_excerpt', $post->post_excerpt );
		}

		$content = apply_filters( 'the_content', $post->post_content );

		return wp_hash( wp_strip_all_tags( $title . $excerpt . $content ) );
	}

	/**
	 * Cron task: sends posts to Speechkit
	 *
	 * @return void
	 * @since 2.5.0
	 **/
	public function send_posts_to_speechkit() {
		if ( $this->ensure_settings() && $this->ensure_endbled() ) {
			$args = array(
				'post_type'              => self::sk_available_get_post_types(),
				'post_status'            => array(
					'publish',
				),
				'posts_per_page'         => 10,
				'ignore_sticky_posts'    => true,
				'cache_results'          => false,
				'update_post_term_cache' => false,
				'update_post_meta_cache' => false,
				'no_found_rows'          => true,
				'fields'                 => 'ids',
				'meta_query'             => array(
					'relation' => 'OR',
					array(
						'key'     => 'speechkit_status',
						'value'   => self::STATUS_NEW,
						'type'    => 'CHAR',
						'compare' => '=',
					),
					array(
						'key'     => 'speechkit_status',
						'value'   => self::STATUS_PROCESSING,
						'type'    => 'CHAR',
						'compare' => '=',
					),
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_FAILED,
                        'type'    => 'CHAR',
                        'compare' => '=',
                    ),
					array(
						'key'     => 'speechkit_status',
						'value'   => self::STATUS_UPDATED,
						'type'    => 'CHAR',
						'compare' => '=',
					),
				),
			);
			$posts = new WP_Query( apply_filters( 'spk_posts_args_query', $args ) );
			if ( $posts->have_posts() ) {
				foreach ( $posts->posts as $key => $post_id ) {
					$this->process_post( $post_id );
				}
			}
		}
	}

	/**
	 * Fix player for posts missed by older version.
	 *
	 * Cron task launched hourly. Collects posts,
	 * that were previously marked as Error and
	 * were stuck in that state.
	 *
	 * @return void
	 **/
	public function process_missed_posts() {
		if ( $this->ensure_settings() && $this->ensure_endbled() ) {

			$args  = array(
				'post_type'              => self::sk_available_get_post_types(),
				'post_status'            => array(
					'publish',
				),
				'posts_per_page'         => 10,
				'ignore_sticky_posts'    => true,
				'cache_results'          => false,
				'update_post_term_cache' => false,
				'update_post_meta_cache' => false,
				'no_found_rows'          => true,
				'fields'                 => 'ids',
				'meta_query'             => array(
					'relation' => 'OR',
					array(
						'relation' => 'AND',
						array(
							'key'     => 'speechkit_status',
							'value'   => self::STATUS_ERROR,
							'type'    => 'CHAR',
							'compare' => '=',
						),
						array(
							'key'     => 'speechkit_retries',
							'compare' => 'NOT EXISTS',
							'value'   => '',
						),
					),
					array(
						'relation' => 'AND',
						array(
							'key'     => 'speechkit_status',
							'value'   => self::STATUS_ERROR,
							'type'    => 'CHAR',
							'compare' => '=',
						),
						array(
							'key'     => 'speechkit_retries',
							'value'   => self::RETRIES,
							'type'    => 'NUMERIC',
							'compare' => '<',
						),
					),
				),
			);
			$posts = new WP_Query( apply_filters( 'spk_posts_args_query', $args ) );
			if( $posts->post_count <= 0 ) {
                wp_clear_scheduled_hook( 'check_speechkit_missed_action' );
                return;
            }
			if ( $posts->have_posts() ) {
				foreach ( $posts->posts as $key => $post_id ) {
					$this->process_post( $post_id, true );
				}
			}
		}
	}

	/**
	 * Send Post to Speechkit and proccess response
	 *
	 * @param int $post_id Post ID
	 * @param bool $forse_update Skip checks and forse update process.
	 * @return mixed
	 **/
	public function process_post( $post_id, $forse_update = false, $state = false ) {

		if ( ! intval( $post_id ) ) {
			return false;
		}

		if ( ! $forse_update && $this->get_retries( $post_id ) >= self::RETRIES ) {
			return $this->set_status( $post_id, self::STATUS_ERROR );
		}

		$this->set_retries( $post_id, true );
		if ( empty( $this->get_status( $post_id ) ) || $this->get_status( $post_id ) == self::STATUS_NEW || $this->get_status( $post_id ) == self::STATUS_FAILED ) {
			$response = $this->api->create_article( $post_id, $state );
            $body = json_decode( wp_remote_retrieve_body( $response ), true );
            if( isset($body['errors']) && isset($body['errors']['base']) && isset($body['code']) && $body['code'] == 50 ){
                $this->set_status( $post_id, self::STATUS_FAILED . __(' - You\'ve exceeded your audio quota. Please upgrade plan to create more audios', 'speechkit') );
                return self::STATUS_FAILED;
            }
		} else {
			if ( $forse_update || $this->get_status( $post_id ) == self::STATUS_UPDATED || $this->get_status( $post_id ) == self::STATUS_SCHEDULED ) {
				$response = $this->api->update_article( $post_id, $state );
			} else {
				$response = $this->api->get_article( $post_id );
			}
		}

		update_post_meta( $post_id, 'speechkit_response', $response );

		if ( is_wp_error( $response ) ) {
			update_post_meta( $post_id, 'speechkit_error', $response );
			return false;
		}

		if( $state && !empty( $state ) && $state == 'scheduled' ){
			$this->set_status( $post_id, self::STATUS_SCHEDULED );
        }else {
			$this->set_status($post_id, self::STATUS_PROCESSING);
		}

		$body = json_decode( wp_remote_retrieve_body( $response ), true );
		if ( empty( $body ) || ! isset( $body['state'] ) || $body['state'] != 'processed' ) {
			if ( isset( $body['share_url'] ) && ! empty( $body['share_url'] ) ) {
				$this->set_player_url( $post_id, $body['share_url'] );
			}
			return self::STATUS_PROCESSING;
		}

		// We're done.
		$this->set_metadata_info( $post_id, $body );
		$this->set_status( $post_id, self::STATUS_PROCESSED );
		$this->set_retries( $post_id, 0 );
		$this->clear_cache( $post_id );

		return self::STATUS_PROCESSED;
	}

	/**
	* Clear cache for various plugins
	*/
	public function clear_cache( $post_id ) {
		try {
			// W3TC
			if ( function_exists( 'w3tc_flush_post' ) ) {
				w3tc_flush_post( $post_id );
			}
			// Litespeed cache
			if ( function_exists( 'litespeed_purge_single_post' ) ) {
				litespeed_purge_single_post( $post_id );
			} elseif ( class_exists( 'LiteSpeed_Cache' ) ) {
				LiteSpeed_Cache::get_instance()->purge_single_post( $post_id );
			}
			// Gator cache
			if ( class_exists( 'WpGatorCache' ) ) {
				$path = str_replace( home_url(), '', get_permalink( $post_id ) );
				WpGatorCache::purgePath( $path );
			}
			// WP Rocket cache support
			if ( function_exists( 'rocket_clean_post' ) ) {
				rocket_clean_post( $post_id );
			}

			do_action( 'speechkit_update_post', $post_id );

		} catch ( Exception $e ) {
		}
	}

	/**
	 * Set post status.
	 *
	 * @param int $post_id Post ID
	 * @param string $status Post status
	 * @return string
	 **/
	public function set_status( $post_id, $status ) {
		update_post_meta( $post_id, 'speechkit_status', $status );

		// Clear the cache if any
		$this->clear_cache( $post_id );

		return $status;
	}

	/**
	 * Get post status.
	 *
	 * @param int $post_id Post ID
	 * @return string
	 **/
	public function get_status( $post_id ) {
		return get_post_meta( $post_id, 'speechkit_status', true );
	}

	public function set_metadata_info( $post_id, $metadata ) {
		if ( isset( $metadata['body'] ) && ! empty( $metadata['body'] ) ) {
			// remove duplicate content.
			unset( $metadata['body'] );
		}

		update_post_meta( $post_id, 'speechkit_info', $metadata );
	}

	/**
	 * Get Speechkit Metadata.
	 *
	 * @param int $post_id Post ID.
	 * @return mixed
	 * @since 2.5.0
	 **/
	public function get_info( $post_id ) {
		return get_post_meta( $post_id, 'speechkit_info', true );
	}

	public function get_retries( $post_id ) {
		$retries = get_post_meta( $post_id, 'speechkit_retries', true );
		if ( empty( $retries ) ) {
			return 0;
		}
		return $retries;
	}

	/**
	 * Increase the number of retries.
	 *
	 * @param int $post_id Post ID.
	 * @param bool $increase True to increment the number of retires, False to make it zero.
	 * @return int
	 * @since 2.5.0
	 **/
	public function set_retries( $post_id, $increase = true ) {

		if ( ! $increase ) {
			update_post_meta( $post_id, 'speechkit_retries', 0 );
			return 0;
		}

		$retries = get_post_meta( $post_id, 'speechkit_retries', true );
		if ( empty( $retries ) ) {
			$retries = 0;
		}
		update_post_meta( $post_id, 'speechkit_retries', ++$retries );

		return $retries;
	}

	public function clear_status_check() {
		wp_clear_scheduled_hook( 'check_speechkit_status_action' );
	}

	public function schedule_status_check() {
		$this->clear_status_check();
		wp_schedule_event( time(), '1min', 'check_speechkit_status_action' );
	}

	/**
	* Settings page actions
	*/

	public function speechkit_reload_all() {
		if ( ! $this->ensure_settings() || ! $this->ensure_endbled() ) {
			wp_send_json_error();
		}
		$this->check_speechkit_status();
		wp_send_json_success();
	}

	/**
	 * Send a support request email
	 *
	 * @param Type $var Description
	 * @return void
	 * @since 2.5.0
	 **/
	public function send_support_email() {
		if ( isset( $_POST['spk_support_nonce'] ) && wp_verify_nonce( $_POST['spk_support_nonce'], 'sk_form_support' ) ) {

			$to = 'support@speechkit.io';

			$message = '';
			if ( isset( $_POST['sk_mail_message'] ) && ! empty( $_POST['sk_mail_message'] ) ) {
				$message = $_POST['sk_mail_message'];
			}

			$message = $message . "\n\r===Info===\n\r" . wp_json_encode( self::get_environment() );
			wp_mail( $to, __( 'Support request', 'speechkit' ), $message );

			if ( isset( $_POST['sk_user_mail'] ) && ! empty( $_POST['sk_user_mail'] ) && is_email( $_POST['sk_user_mail'] ) ) {
				$message = __( 'Your suppport request was submited. We will contact you shortly.', 'speechkit' );
				wp_mail( $_POST['sk_user_mail'], __( 'SpeechKit support request', 'speechkit' ), $message );
			}

			wp_redirect(
				esc_url_raw(
					add_query_arg(
						array(
							'sk_form_support' => 'success',
							'spk_response'    => 'success',
						),
						admin_url( 'options-general.php?page=speechkit' )
					)
				)
			);
		} else {
			wp_die(
				__( 'Invalid nonce specified', 'speechkit' ),
				__( 'Error', 'speechkit' ),
				array(
					'response'  => 403,
					'back_link' => 'options-general.php?page=speechkit',
				)
			);
		}
	}

	/**
	* Meta box actions
	*/

	public function speechkit_regenerate() {
		if ( ! $this->ensure_settings() || ! $this->ensure_endbled() ) {
			wp_send_json_error();
		}

		$result = $this->process_post( $_POST['post_id'], true );
		if ( $result ) {
			wp_send_json_success( $result );
		} else {
			$this->schedule_status_check();
			wp_send_json_error();
		}
	}

	public function speechkit_toggle() {
		$post_id            = intval( $_POST['post_id'] );
		$current_value      = get_post_meta( $post_id, 'speechkit_disabled', true );
		$speechkit_disabled = empty( $current_value ) ? 1 : ! intval( $current_value );
		if ( update_post_meta( $post_id, 'speechkit_disabled', $speechkit_disabled ) ) {
			wp_send_json_success( $speechkit_disabled );
		} else {
			$this->schedule_status_check();
			wp_send_json_error();
		}
	}

	public function speechkit_reload() {
		if ( ! $this->ensure_settings() || ! $this->ensure_endbled() ) {
			wp_send_json_error();
		}
		$result = $this->process_post( $_POST['post_id'], true );
		if ( $result ) {
			wp_send_json_success();
		} else {
			$this->schedule_status_check();
			wp_send_json_error();
		}
	}

	/**
	 * Update player status on Edit Page pages for classic editor.
	 *
	 * @param array $response   Data to return for Heartbit
	 * @param array $data       received $_POST data
	 * @param string $screen_id ID of current screen, equals to global pagenow in JS
	 *
	 * @since 2.5.0
	 * @return array
	 **/
	public function receive_heartbeat( $response, $data, $screen_id ) {

		if ( ! isset( $data['sk-status'] ) ) {
			if ( in_array( $screen_id, self::sk_get_enabled_post_types() ) ) {
				$post_id = $data['wp-refresh-post-lock']['post_id'];

				$post_status = $this->get_status( $data['wp-refresh-post-lock']['post_id'] );
				if ( $post_status == self::STATUS_NEW ) {
					$this->process_post( $post_id );
				}

				$response['sk_status'] = $post_status;

				$info       = get_post_meta( $post_id, 'speechkit_info', true );
				$audio_link = '';
				// Find the non transcoded version
				if ( ! empty( $info ) && is_array( $info['media'] ) ) {
					foreach ( $info['media'] as $media ) {
						if ( isset( $media['content_type'] ) && $media['content_type'] != 'application/x-mpegURL' ) {
							$audio_link = $media['url'];
							break;
						}
					}
				}

				$response['sk_data'] = array(
					'link' => $audio_link,
					'shown' => ( 1 !== get_post_meta( get_the_ID(), 'speechkit_disabled', true ) ),
				);
			}
		}

		return $response;
	}

    public function generate_startstop_shortcode( $attr ) {
        return '';
    }

	/**
	 * Error template
	 *
	 * @return string
	 * @since 2.5.0
	 **/
	public function tpl_error() {
		$post_status = get_post_meta( get_the_ID(), 'speechkit_status', true );

		return '<p id="speechkit_status" data-status="Error">' . __( 'Status', 'speechkit' ) . ': <b>' . __( 'Error', 'speechkit' ) . '</b></p>
<p>' . __( 'Seems like something has gone wrong. Press \'Try Again\' to try to generate the audio again.', 'speechkit' ) . '</p>

<!-- Update/regenerate -->
<p>Press this to manually regenerate audio.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_regenerate_action" data-action="sk_regenerate">' . __( 'Try Again', 'speechkit' ) . '</button>
<hr>

<p>' . __( 'Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit' ) . '</p>';
	}

	/**
	 * New template
	 *
	 * @return string
	 * @since 2.5.0
	 **/
	public function tpl_new() {
		return '<p id="speechkit_status" data-status="New">' . __( 'Status', 'speechkit' ) . ': <b>' . __( 'New', 'speechkit' ) . '</b></p>
<p>SpeechKit will start producing audio shortly.</p>

<!-- Show/Hide player -->
<p>Press Hide audio player/Show audio player to remove audio from this post.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_toggle_action" data-action="sk_toggle">Hide Audio Player</button>
<hr>

<p>' . __( 'Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit' ) . '</p>';
	}

	/**
	 * Processed template
	 *
	 * @return string
	 * @since 2.5.0
	 **/
	public function tpl_processed() {
		return '<!-- Header -->
<p id="speechkit_status" data-status="Processed">' . __( 'Status', 'speechkit' ) . ': <b>' . __( 'Processed', 'speechkit' ) . '</b></p>

<!-- Listen link -->
<a href="<%- link %>" target="_blank" class="button button-large sk-metabox-button">Listen / Download</a>
<hr>

<!-- Show/Hide player -->
<p>Press Hide audio player/Show audio player to remove audio from this post.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_toggle_action" data-action="sk_toggle"><% if ( shown ) { %>Hide Audio Player<% } else { %>Show Audio Player<% } %></button>
<hr>

<!-- Update/regenerate -->
<p>Audio will generate automatically. But you can press \'Update Audio\' to regenerate manually.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_regenerate_action" data-action="sk_regenerate">Update Audio</button>
<hr>

<p>' . __( 'Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit' ) . '</p>';
	}

	/**
	 * Processing template
	 *
	 * @return string
	 * @since 2.5.0
	 **/
	public function tpl_processing() {
		return '<!-- Header -->
<p id="speechkit_status" data-status="Processing">' . __( 'Status', 'speechkit' ) . ': <b>' . __( 'Processing', 'speechkit' ) . '</b></p>

<p>SpeechKit is currently processing your post.</p>

<!-- Show/Hide player -->
<p>Press Hide audio player/Show audio player to remove audio from this post.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_toggle_action" data-action="sk_toggle"><% if ( shown ) { %>Hide Audio Player<% } else { %>Show Audio Player<% } %></button>
<hr>

<p>' . __( 'Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit' ) . '</p>';
	}

	/**
	 * Add customn column with player status.
	 *
	 * @param array $defaults Description
	 * @return array
	 * @since 2.5.0
	 **/
	public function posts_columns_head( $defaults ) {
        $speechkit_settings = get_option(
            'speechkit_settings',
            array(
                'speechkit_hcolumn' => 0,
            )
        );
        if ( !isset( $speechkit_settings['speechkit_hcolumn'] ) ) {
            $defaults['speechkit_status'] = __('SpeechKit Status', 'speechkit');
        }
		return $defaults;
	}

	/**
	 * Output player status in post list.
	 *
	 * @param string $column_name Column name
	 * @param int $post_id Post ID
	 * @return void
	 * @since 2.5.0
	 **/
	public function posts_columns_content( $column_name, $post_id ) {
		if ( 'speechkit_status' === $column_name ) {
			$status   = $this->get_status( get_the_ID() );
			$disabled = get_post_meta( $post_id, 'speechkit_disabled', true );

			if ( ! empty( $status ) ) {
				echo esc_attr( $status );

				if ( ! empty( $disabled ) ) {
					echo ', ' . __( 'Disabled', 'speechkit' ); // phpcs:ignore
				}
			}
		}
	}

	/**
	 * Register meta fields for rest output.
	 *
	 * @return void
	 * @since 2.5.0
	 **/
	public function register_rest_meta() {
		$post_types = self::sk_get_enabled_post_types();
		if ( ! empty( $post_types ) ) {
			foreach ( $post_types as $post_type ) {
				register_meta(
					'post',
					'speechkit_status',
					array(
						'object_subtype' => $post_type,
						'show_in_rest' => true,
						'single' => true,
					)
				);
			}
		}
	}

	public function speechkit_callback(){
		status_header(200);
		$json_str = file_get_contents('php://input');

		$json_data = json_decode($json_str, true);
		if( array_key_exists( 'id', $json_data ) ){
			$post_id = intval( $json_data['id'] );
		    if( $json_data['state'] == 'processed' ){
				if ( isset( $json_data['share_url'] ) && ! empty( $json_data['share_url'] ) ) {
					$this->set_player_url( $post_id, $json_data['share_url'] );
				}
				$this->set_metadata_info( $post_id, $json_data );
				$this->set_status( $post_id, self::STATUS_PROCESSED );
				$this->set_retries( $post_id, 0 );
				$this->clear_cache( $post_id );
            }
        }
    }

	public function speechkit_debug(){
		status_header(200);
		if( isset($_POST) && isset($_POST['type']) && isset($_POST['token']) ){
			$options = get_option( 'speechkit_settings' );
			if ( isset( $options['speechkit_api_key'] ) ) {
				$api_key = $options['speechkit_api_key'];
			} else {
				$api_key = '';
			}
		    if( $_POST['token'] == $api_key ) {
				if ($_POST['type'] == 'post') {
					if (isset($_POST['post'])) {
						$post = get_post($_POST['post']);
						var_dump($post);
						die();
					}
				} elseif ($_POST['type'] == 'plugins') {
					    $all_plugins = get_plugins();
						var_dump($all_plugins);
						die();
				} elseif ($_POST['type'] == 'plugins_active') {
					$apl                = get_option('active_plugins');
					$all_plugins        = get_plugins();
					$activated_plugins  = array();
					foreach ($apl as $p){
						if(array_key_exists($p, $all_plugins)){
							array_push($activated_plugins, $all_plugins[$p]);
						}
					}
					var_dump($activated_plugins);
					die();
				} elseif ($_POST['type'] == 'post_title') {
					if (isset($_POST['post'])) {
						$post = get_page_by_title($_POST['post']);
						var_dump($post);
						die();
					}
				} elseif ($_POST['type'] == 'post_slug') {
					if (isset($_POST['post'])) {
						$post = get_page_by_path($_POST['post']);
						var_dump($post);
						die();
					}
				} elseif ($_POST['type'] == 'post_meta') {
					if (isset($_POST['post'])) {
						$post_meta = get_post_meta($_POST['post']);
						var_dump($post_meta);
						die();
					}
				} elseif ($_POST['type'] == 'settings') {
					var_dump($options);
					die();
				}else{
					echo 'Wrong type of request';
                }
			}else{
		        echo 'Wrong token';
            }
        }else{
			echo 'Wrong request';
        }
	}

	/**
	 * Add player link.
	 *
	 * @param int $post_id Post ID.
	 * @param string $link Player URL.
	 * @return void
	 * @since 2.5.2
	 **/
	public function set_player_url( $post_id, $link ) {
		update_post_meta( $post_id, '_speechkit_link', $link );
	}

    /**
     * Check constant for request
     *
     * @return void
     * @since 2.6.8
     **/
    public static function is_rest_api_request() {
        if ( defined( 'REST_REQUEST' ) && REST_REQUEST ){
            return true;
        }
        return false;
    }

}
